#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys, re, os, apt, time
from PyQt4 import QtGui,QtCore
from leinstallpkgs.gui.progressUI import Ui_mainWindow
from apt.progress import base

class Qt4InstallProgress(QtCore.QObject, base.InstallProgress):
    """ Installation progress with Qt4 signals"""

    INSTALL_TIMEOUT = 5*60

    def __init__(self):
        base.InstallProgress.__init__(self)
        QtCore.QObject.__init__(self)
        self.apt_status = -1
        self.finished = False
        self.time_last_update = time.time()

    def error(self, pkg, errormsg):
        """ Called when a error is detected during the install """

        #base.InstallProgress.error(self, pkg, errormsg)
        self.emit(QtCore.SIGNAL("statusError()"))
        print "error", self, pkg, errormsg

    def processing(self, pkg, stage):
        """ This method is called just before a processing stage starts.
        The parameter pkg is the name of the package and the parameter
        stage is one of the stages listed in the dpkg manual under the
        status-fd option, i.e. "upgrade", "install" (both sent before
        unpacking), "configure", "trigproc", "remove", "purge". """

        #base.InstallProgress.processing(self, pkg, state)
        self.emit(QtCore.SIGNAL("statusChanged(PyQt_PyObject)", pkg))
        print "Processing", self, pkg, stage

    def start_update(self):
        """ Called when the update starts. """

        print "start_update"
        self.emit(QtCore.SIGNAL("statusStarted()"))

    def run(self, obj):
        """ Run. """

        self.finished = False
        return base.InstallProgress.run(self, obj)
        print "run"

    def finish_update(self):
        """ Called when the update finished. """

        self.emit(QtCore.SIGNAL("statusFinished()"))
        print "finish_update"

    def processing(self, pkg, stage):
        """ Called when entering a new stage in dpkg. """

        # We have no porcentage or alike send -1 to let the bar pulse.
        self.emit(QtCore.SIGNAL("statusChanged(PyQt_PyObject)"), pkg)
        print "processing"

    def status_change(self, pkg, percent, status):
        """ This method implements progress reporting for package
        installation by APT and may be extended to dpkg at a later time.
        This method takes two parameters: The parameter percent is a float
        value describing the overall progress and the parameter status is a
        string describing the current status in an human-readable manner """

        base.InstallProgress.status_change(self, pkg, percent, status)
        self.time_last_update = time.time()
        self.emit(QtCore.SIGNAL("statusChanged(PyQt_PyObject,PyQt_PyObject)"),
                                status, percent)
        print "status_change"

    def update_interface(self):
        """ Called periodically to update the interface. """

        base.InstallProgress.update_interface(self)
        if self.time_last_update + self.INSTALL_TIMEOUT < time.time():
            self.emit(QtCore.SIGNAL("statusTimeout()"))

    def child_exited(self, term, pid, status):
        """ Called when a child process exits """
        self.apt_status = os.WEXITSTATUS(status)
        self.finished = True
        print "child_exited"

class Qt4AcquireProgress(QtCore.QObject, base.AcquireProgress):
    """ A Fetch Progress with Qt4 signals"""

    def __init__(self):
        base.AcquireProgress.__init__(self)
        QtCore.QObject.__init__(self)
        self._continue = True

    def cancel(self):
        self._continue = False

    def done(self, item):
        """Invoked when an item is successfully and completely fetched """
        base.AcquireProgress.done(self, item)
        print "Done:", self, item

    def fail(self, item):
        base.AcquireProgress.fail(self, item)
        self.emit(QtCore.SIGNAL("statusError()"))
        print "Fail", self, item

    def fetch(self, item):
        base.AcquireProgress.fetch(self, item)
        print "Fetch", self, item

    def pulse(self, owner):
        base.AcquireProgress.pulse(self, owner)
        current_item = self.current_items + 1
        if current_item > self.total_items:
            current_item = self.total_items
        if self.current_cps > 0:
            text = ("Baixando arquivo %(current)li de %(total)li a "
                      "%(speed)s/s") % \
                      {"current": current_item,
                       "total": self.total_items,
                       "speed": apt.apt_pkg.size_to_str(self.current_cps)}
        else:
            text = ("Baixando arquivo %(current)li de %(total)li") % \
                      {"current": current_item,
                       "total": self.total_items}

        percent = (((self.current_bytes + self.current_items) * 100.0) /
                        float(self.total_bytes + self.total_items))

        self.emit(QtCore.SIGNAL("statusChanged(PyQt_PyObject,PyQt_PyObject)"),
                                text, percent)
        return self._continue


    def start(self):
        base.AcquireProgress.start(self)
        self.emit(QtCore.SIGNAL("statusStarted()"))
        print "Start", self

    def stop(self):
        base.AcquireProgress.stop(self)
        self.emit(QtCore.SIGNAL("statusFinished()"))
        print "Stop", self

class Qt4OpProgress(QtCore.QObject, base.OpProgress):
    """ Operation progress with Qt4 signals. """

    def __init__(self):
        base.OpProgress.__init__(self)
        QtCore.QObject.__init__(self)

    def done(self):
        """ Called when all operation have finished. """
        base.OpProgress.done(self)
        self.emit(QtCore.SIGNAL("statusFinished()"))
        print "Done", self

    def update(self, percent=None):
        """ Called to update the percentage done. """
        base.OpProgress.update(self, percent)
        self.emit(QtCore.SIGNAL("statusChanged(PyQt_PyObject,PyQt_PyObject)"),
                            self.op, self.percent)

class Qt4AptProgress(QtCore.QObject):
    """ Graphical progress for installation/fetch/operations.
    This widget provides a progress bar, a terminal and a status bar for
    showing the progress of package manipulation tasks. """

    def __init__(self, parent=None):
        QtCore.QObject.__init__(self)
        self._dialog = Qt4ProgressGui(parent)
        self._progress_open = Qt4OpProgress()
        self._progress_acquire = Qt4AcquireProgress()
        self._progress_install = Qt4InstallProgress()

        # If something goes wrong this flag is set to False and the process
        # stops
        self._continue = True

        # Some defines to get the return result easily
        self.Accepted = self._dialog.Accepted
        self.Rejected = self._dialog.Rejected

        # Connect signals
        progress_op_signals = {
                    "statusChanged(PyQt_PyObject,PyQt_PyObject)":
                        self._on_status_changed,
                    "statusStarted()": self._on_status_started,
                    "statusFinished()": self._on_status_finished }

        acquire_signals = {
                    "statusChanged(PyQt_PyObject,PyQt_PyObject)":
                        self._on_status_changed,
                    "statusError()": self._on_acquire_error,
                    "statusStarted()": self._on_acquire_started,
                    "statusFinished()": self._on_acquire_finished }

        progress_install_signals = {
                    "statusChanged(PyQt_PyObject,PyQt_PyObject)":
                        self._on_status_changed,
                    "statusStarted()": self._on_status_started,
                    "statusFinished()": self._on_status_finished,
                    "statusTimeout()": self._on_status_timeout,
                    "statusError()": self._on_status_timeout,
                    "statusConffile()": self._on_status_timeout }

        # Dialog signals
        self._dialog.buttonOK.connect(self._dialog.buttonCancel,
                                      QtCore.SIGNAL("clicked()"),
                                      self.cancel_download)

        self.connect(self, QtCore.SIGNAL("externalError(PyQt_PyObject)"),
                     self._externalError)

        self._connect_signals(self._progress_open, progress_op_signals)
        self._connect_signals(self._progress_acquire, acquire_signals)
        self._connect_signals(self._progress_install, progress_install_signals)

    def _on_acquire_error(self):
        text = "Ocorreu um erro ao baixar o(s) pacote(s)."
        self._externalError(text)

    def _on_acquire_started(self):
        self.dialog().setCancel(True)
        self.dialog().setOk(False)

    def _on_acquire_finished(self):
        self.dialog().setCancel(False)

    def _connect_signals(self, obj, signals):
        """ Connect obj signals """
        for signal, slot in signals.items():
            obj.connect(obj,QtCore.SIGNAL(signal),slot)

    def open(self):
        """Return the cache opening progress handler."""
        return self._progress_open

    def install(self):
        """Return the install progress handler."""
        return self._progress_install

    def acquire(self):
        """Return the acquire progress handler."""
        return self._progress_acquire

    def dialog(self):
        return self._dialog

    def error(self, text):
        """ This method stops execution and print text as error message """
        self.emit(QtCore.SIGNAL("externalError(PyQt_PyObject)"),text)

    def _externalError(self, text):
        """ This method stops execution and print text as error message """
        self._continue = False
        self._dialog.setPkgInstallingText(text)
        self._dialog.pulse(100.0)
        self._dialog.addText(text)
        self._dialog.setStatusLabelText("ERRO!", "red")
        self._dialog.finished()

    def _on_status_started(self):
        """Called when something starts."""
        self._on_status_changed("Iniciando...", 0)

    def _on_status_changed(self, status, percent=0.0):
        """Called when the status changed."""
        if self._continue:
            self._dialog.setPkgInstallingText(status)
            self._dialog.addText(status)
            if percent is None or percent == -1:
                pass
            else:
                self._dialog.pulse(percent)

    def _on_status_timeout(self):
        """Called when timeout happens."""
        #self._expander.set_expanded(True)
        print "TIMEOUT"
        pass

    def _on_status_finished(self, progress=0):
        """Called when something finished."""
        self._on_status_changed("Pronto.", 100)
        self._dialog.finished()

    def cancel_download(self):
        """Cancel a currently running download."""
        print "cancelando"
        if self._dialog.shouldCancel():
            self.acquire().cancel()
            self._dialog.hide()
            self._dialog.showInfo(u"Operação Cancelada")

    def show(self):
        self._dialog.show()

    def exec_(self):
        return self._dialog.exec_()

class Qt4ProgressGui(QtGui.QDialog, Ui_mainWindow):
    """ Opens a QDialog with a progress bar and a text area """

    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)

        # Set up the user interface from Designer
        self.setupUi(self)

        self.setOk(False)
        self.setCancel(True)

    def setCancel(self, opt):
        self.buttonCancel.setEnabled(opt)

    def setStatusLabelText(self, text, color="red"):
        self.statusLabel.setStyleSheet("QLabel { color : %s; }" % color)
        self.statusLabel.setText(str(text))

    def setOk(self, opt):
        self.buttonOK.setEnabled(opt)

    def showInfo(self, msg):
        QtGui.QMessageBox.information(self, u"Edubar", msg)

    def ask(self, msg, b1, b2):
        if QtGui.QMessageBox.question(self, u"Edubar", msg, b1, b2) == 0:
            return True
        else:
            return False

    def shouldCancel(self):
        msg = u"Deseja cancelar o processo de instalação?"
        return self.ask(msg, u"Sim", u"Não")

    def finished(self):
        self.buttonOK.setEnabled(True)
        self.buttonCancel.setEnabled(False)

    def pulse(self, percent):
        self.progressBar.setValue(int(percent))

    def addText(self, text):
        self.progressText.appendPlainText(str(text))

    def setPkgInstallingText(self, pkgname):
        self.pkgInstallingLabel.setText(str(pkgname))

    #def done(self):
    # """ Called when the user clicks on the OK button  """
    #    self.addText("Pronto")
    #    self.setPkgInstallingText("Pronto")
